#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.location.interfaces import ILocationTarget, ILocationInfo
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_form.interfaces.form import IWidgetForm, IInnerSubForm
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_PERMISSION, VIEW_SYSTEM_PERMISSION
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.shared.common.zmi import WfSharedContentHeaderAdapter
from pyams_form.form import AJAXEditForm, InnerEditForm
from pyams_form.group import NamedWidgetsGroup
from pyams_form.schema import ResetButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem, MenuDivider
from pyams_utils.adapter import adapter_config
from pyams_utils.property import cached_property
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, alsoProvides, Interface

from onf_website import _


@viewlet_config(name='location.divider', context=ILocationTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_PERMISSION, weight=299)
class LocationMenuDivider(MenuDivider):
    """Location menu divider"""


@viewlet_config(name='location.menu', context=ILocationTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=VIEW_SYSTEM_PERMISSION, weight=300)
class LocationMenu(MenuItem):
    """Location menu"""

    label = _("Location...")
    icon_class = 'fa-map-marker'
    url = '#location.html'


class ILocationEditGroup(Interface):
    """Inner location edit group"""


class ILocationEditButtons(Interface):
    """Location form buttons"""

    reset = ResetButton(name='reset', title=_("Reset"))
    submit = button.Button(name='submit', title=_("Submit"))


@pagelet_config(name='location.html', context=ILocationTarget, layer=IPyAMSLayer, permission=VIEW_SYSTEM_PERMISSION)
@implementer(IWidgetForm, IInnerPage)
class LocationEditForm(AdminEditForm):
    """Location edit form"""

    legend = _("Content location")

    @property
    def fields(self):
        if ILocationInfo(self.context).can_inherit:
            fields = field.Fields(ILocationInfo).select('no_inherit')
        else:
            fields = field.Fields(Interface)
        return fields

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(ILocationEditButtons)
        else:
            return button.Buttons(Interface)

    ajax_handler = 'location.json'

    def update(self):
        super(LocationEditForm, self).update()
        if ILocationInfo(self.context).can_inherit:
            group = NamedWidgetsGroup(self, 'location', self.widgets,
                                      ('no_inherit', ),
                                      legend=_("Don't inherit parent location"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_field=ILocationInfo['no_inherit'])
        else:
            group = NamedWidgetsGroup(self, 'location', self.widgets, (), css_class='inner')
        alsoProvides(group, ILocationEditGroup)
        [subform.update() for subform in group.subforms]
        self.add_group(group)


@view_config(name='location.json', context=ILocationTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class LocationAJAXEditForm(AJAXEditForm, LocationEditForm):
    """Location edit form, JSON renderer"""

    @cached_property
    def forms(self):
        if ILocationInfo(self.context).can_inherit:
            group = NamedWidgetsGroup(self, 'location', self.widgets,
                                      ('no_inherit', ),
                                      legend=_("Don't inherit parent location"),
                                      css_class='inner',
                                      switch=True,
                                      checkbox_switch=True,
                                      checkbox_field=ILocationInfo['no_inherit'])
        else:
            group = NamedWidgetsGroup(self, 'location', self.widgets, (), css_class='inner')
        alsoProvides(group, ILocationEditGroup)
        [subform.update() for subform in group.subforms]
        self.add_group(group)
        return super(LocationAJAXEditForm, self).forms + group.subforms


@adapter_config(name='main', context=(ILocationTarget, IPyAMSLayer, ILocationEditGroup), provides=IInnerSubForm)
class LocationMainEditSubform(InnerEditForm):
    """Location main edit form"""

    legend = None
    main_group_legend = _("Departments and countries")

    fields = field.Fields(ILocationInfo).select('countries', 'departments', 'structures')
    weight = 1


@adapter_config(name='address', context=(ILocationTarget, IPyAMSLayer, ILocationEditGroup), provides=IInnerSubForm)
class LocationAddressEditSubform(InnerEditForm):
    """Location address edit form"""

    legend = None
    main_group_legend = _("Address")

    fields = field.Fields(ILocationInfo).select('address', 'postal_code', 'city', 'situation')
    weight = 2

    def updateWidgets(self, prefix=None):
        super(LocationAddressEditSubform, self).updateWidgets(prefix)
        self.widgets['address'].label_css_class = 'textarea'
        self.widgets['postal_code'].input_css_class = 'col-md-2'


@adapter_config(name='forests', context=(ILocationTarget, IPyAMSLayer, ILocationEditGroup), provides=IInnerSubForm)
class LocationForestsEditSubform(InnerEditForm):
    """Location forests edit form"""

    legend = None
    main_group_legend = _("Forests")

    fields = field.Fields(ILocationInfo).select('natural_region', 'forests', 'forests_label')
    weight = 3


@adapter_config(name='coordinates', context=(ILocationTarget, IPyAMSLayer, ILocationEditGroup), provides=IInnerSubForm)
class LocationCoordinatesEditSubform(InnerEditForm):
    """Location coordinates edit form"""

    legend = None
    main_group_legend = _("GPS coordinates")

    fields = field.Fields(ILocationInfo).select('projection', 'longitude', 'latitude', 'altitude')
    weight = 4

    def updateWidgets(self, prefix=None):
        super(LocationCoordinatesEditSubform, self).updateWidgets(prefix)
        self.widgets['longitude'].input_css_class = 'col-md-2'
        self.widgets['latitude'].input_css_class = 'col-md-2'
        self.widgets['altitude'].input_css_class = 'col-md-2'


@adapter_config(context=(ILocationTarget, IAdminLayer, LocationEditForm), provides=IPageHeader)
class LocationHeaderAdapter(WfSharedContentHeaderAdapter):
    """Shared content location header adapter"""

    icon_class = 'fa fa-fw fa-map-marker'
