#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.component.location.interfaces import ILocationInfo
from onf_website.root.interfaces import ISiteRootToolsConfiguration
from pyams_utils.interfaces.site import ISiteGenerations
from zope.site.interfaces import INewLocalSite

# import packages
from hypatia.text.lexicon import Lexicon
from onf_website.reference import ReferencesManager
from onf_website.reference.location.agency import AgencyTable
from onf_website.reference.location.country import CountryTable
from onf_website.reference.location.department import DepartmentTable
from onf_website.reference.location.region import RegionTable
from onf_website.reference.location.territory import TerritoryTable
from onf_website.reference.location.unit import UnitTable
from onf_website.reference.target import TargetTable
from pyams_catalog.index import KeywordIndexWithInterface
from pyams_catalog.nltk import NltkFullTextProcessor
from pyams_catalog.site import check_required_indexes
from pyams_utils.registry import utility_config
from pyams_utils.site import check_required_utilities
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.lifecycleevent import ObjectCreatedEvent


def get_fulltext_lexicon():
    return Lexicon(NltkFullTextProcessor(language='french'))


REQUIRED_UTILITIES = ()


REQUIRED_INDEXES = (('countries', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                              'discriminator': 'countries_index'}),
                    ('departments', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                                'discriminator': 'departments_index'}),
                    ('structures', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                               'discriminator': 'structures'}),
                    ('forests', KeywordIndexWithInterface, {'interface': ILocationInfo,
                                                            'discriminator': 'forests'}))


@subscriber(INewLocalSite)
def handle_new_local_site(event):
    """Check for required utilities when a site is created"""
    site = event.manager.__parent__
    check_required_utilities(site, REQUIRED_UTILITIES)
    check_required_indexes(site, REQUIRED_INDEXES)


@utility_config(name='ONF website', provides=ISiteGenerations)
class WebsiteGenerationsChecker(object):
    """ONF website generations checker"""

    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)
        check_required_indexes(site, REQUIRED_INDEXES)
        registry = get_current_registry()
        tools_configuration = ISiteRootToolsConfiguration(site, None)
        if tools_configuration is not None:
            # check references tables
            tables_name = tools_configuration.tables_name or \
                          registry.settings.get('onf_website.config.tables_name', 'references')
            if tables_name not in site:
                tables_manager = ReferencesManager()
                registry.notify(ObjectCreatedEvent(tables_manager))
                tables_manager.title = {'en': "References tables",
                                        'fr': "Tables de références"}
                tables_manager.short_name = {'en': "References tables",
                                             'fr': "Tables de références"}
                site[tables_name] = tables_manager
                tools_configuration.tables_name = tables_name
            else:
                tables_manager = site[tables_name]
            # check references tables
            for attr, name, factory in (('country_table_name', 'countries', CountryTable),
                                        ('region_table_name', 'regions', RegionTable),
                                        ('department_table_name', 'departments', DepartmentTable),
                                        ('territory_table_name', 'territories', TerritoryTable),
                                        ('agency_table_name', 'agencies', AgencyTable),
                                        ('unit_table_name', 'units', UnitTable),
                                        ('target_table_name', 'targets', TargetTable)):
                table_name = getattr(tools_configuration, attr, None) or \
                             registry.settings.get('onf_website.config.' + attr, name)
                if table_name not in tables_manager:
                    table = factory()
                    registry.notify(ObjectCreatedEvent(table))
                    tables_manager[table_name] = table
                    setattr(tools_configuration, attr, table_name)
